@@ -16,6 +16,6 @@ plot(t,s, linewidth=4)
 axis([-.2, 2.2, -1.2, 1.2])
 
 grid(True)
-set(gca(), xticklabels=[], yticklabels=[])
+setp(gca(), xticklabels=[], yticklabels=[])
 #savefig('../images/matplotlib.svg', facecolor=0.75)
 
