@@ -10,7 +10,7 @@ xlabel('time (s)')
 ylabel('voltage (mV)')
 title('About as simple as it gets, folks')
 grid(True)
-savefig('simple_plot.png')
-savefig('simple_plot.ps')
+#savefig('simple_plot.png')
+savefig('simple_plot')
 
 show()
