@@ -71,9 +71,12 @@ _Plotting commands
   spy2     - plot sparsity pattern using image
   stem     - make a stem plot
   subplot  - make a subplot (numrows, numcols, axesnum)
+  subplots_adjust - change the params controlling the subplot positions of current figure
+  subplot_tool - launch the subplot configuration tool
   table    - add a table to the plot
   text     - add some text at location x,y to the current axes
   thetagrids - customize the radial theta grids and labels for polar
+
   title    - add a title to the current axes
   xlim     - set/get the xlimits
   ylim     - set/get the ylimits
@@ -187,15 +190,13 @@ John D. Hunter <jdhunter@ace.bsd.uhicago.edu>
 Most of the other commands are from Numeric, MLab and FFT, with the
 exception of those in mlab.py provided by matplotlib.
 """
-import warnings
+import sys, warnings
 import cm
 import _pylab_helpers
 import mlab  #so I can override hist, psd, etc...
 
 from axes import Axes, PolarAxes
 import backends
-from backends import new_figure_manager, draw_if_interactive, show
-
 from cbook import flatten, is_string_like, exception_to_str, popd, \
      silent_list, iterable, enumerate
 from colors import normalize
@@ -207,6 +208,13 @@ from backend_bases import FigureCanvasBase
 from artist import ArtistInspector, getp, get
 from artist import setp as _setp
 
+# a hack to keep old versions of ipython working with mpl after bug
+# fix #1209354
+if 'IPython.Shell' in  sys.modules:
+    from backends import new_figure_manager, draw_if_interactive, show
+else:
+    from backends import pylab_setup
+    new_figure_manager, draw_if_interactive, show = pylab_setup()
 
 
 
@@ -215,6 +223,9 @@ from lines import Line2D
 from text import Text
 from patches import Polygon, Rectangle, Circle, Arrow
 from transforms import blend_xy_sep_transform
+from widgets import SubplotTool, Button, Slider, Widget
+
+import numerix as nx
 
 # catch more than an import error here, since the src could fail too,
 # eg a bad pytz install.  I don't want to break all of matplotlib for
@@ -252,7 +263,11 @@ from numerix import Int8, UInt8, Int16, UInt16, Int32, UInt32, Float32, Float64,
 from matplotlib.numerix.fft import fft
 from matplotlib.numerix.linear_algebra import inverse, eigenvectors
 
-from matplotlib.numerix.mlab import rand,randn,eye,tri,diag,fliplr,flipud,rot90,tril,triu,ptp,mean,msort,median,std,cumsum,prod,cumprod,trapz,diff,cov,corrcoef,squeeze,kaiser,blackman,bartlett,hanning,hamming,sinc,eig,svd,angle,roots,amin,  amax
+#from matplotlib.numerix.mlab import rand,randn,eye,tri,diag,fliplr,flipud,rot90,tril,triu,ptp,mean,msort,median,std,cumsum,prod,cumprod,trapz,diff,cov,corrcoef,squeeze,kaiser,blackman,bartlett,hanning,hamming,sinc,eig,svd,angle,roots,amin,  amax
+
+pymin, pymax = min, max
+from matplotlib.numerix.mlab import *
+min, max = pymin, pymax
 
 from matplotlib.mlab import linspace, window_hanning, window_none, conv, detrend, detrend_mean, detrend_none, detrend_linear, corrcoef, polyfit, polyval, vander, entropy, normpdf, levypdf, find, trapz, prepca, fix, rem, norm, orth, rank, sqrtm, prctile, center_matrix, meshgrid, rk4, exp_safe, amap, sum_flat, mean_flat, rms_flat, l1norm, l2norm, norm, frange, diagonal_matrix, base_repr, binary_repr, log2, ispower2, bivariate_normal
 
@@ -295,25 +310,64 @@ def _shift_string(s):
     return ''.join(lines)
 
 
-def colorbar(tickfmt='%1.1f', cax=None, orientation='vertical'):
+def colorbar(mappable = None,
+             cax=None,
+             orientation='vertical',
+             tickfmt='%1.1f',
+             cspacing='proportional',
+             clabels=None,
+             drawedges=False,
+             edgewidth=0.5,
+             edgecolor='k'):
     """
-    Create a colorbar for current image
-    
+    Create a colorbar for mappable; if mappable is None,
+    use current image.
+
     tickfmt is a format string to format the colorbar ticks
 
     cax is a colorbar axes instance in which the colorbar will be
     placed.  If None, as default axesd will be created resizing the
     current aqxes to make room for it.  If not None, the supplied axes
     will be used and the other axes positions will be unchanged.
-    
+
     orientation is the colorbar orientation: one of 'vertical' | 'horizontal'
+
+    cspacing controls how colors are distributed on the colorbar.
+    if cspacing == 'linear', each color occupies an equal area
+    on the colorbar, regardless of the contour spacing.
+    if cspacing == 'proportional' (Default), the area each color
+    occupies on the the colorbar is proportional to the contour interval.
+    Only relevant for a Contour image.
+
+    clabels can be a sequence containing the
+    contour levels to be labelled on the colorbar, or None (Default).
+    If clabels is None, labels for all contour intervals are
+    displayed. Only relevant for a Contour image.
+
+    if drawedges == True, lines are drawn at the edges between
+    each color on the colorbar. Default False.
+
+    edgecolor is the line color delimiting the edges of the colors
+    on the colorbar (if drawedges == True). Default black ('k')
+
+    edgewidth is the width of the lines delimiting the edges of
+    the colors on the colorbar (if drawedges == True). Default 0.5
+
     return value is the colorbar axes instance
     """
-    mappable = gci()
-    ret = gcf().colorbar(mappable, tickfmt, cax, orientation)
+    if mappable is None:
+        mappable = gci()
+    ret = gcf().colorbar(mappable, cax = cax,
+                         orientation = orientation,
+                         tickfmt = tickfmt,
+                         cspacing=cspacing,
+                         clabels=clabels,
+                         drawedges=drawedges,
+                         edgewidth=edgewidth,
+                         edgecolor=edgecolor)
     draw_if_interactive()
     return ret
-    
+
 def colors():
     """
     This is a do nothing function to provide you with help on how
@@ -466,12 +520,12 @@ def get_current_fig_manager():
 def connect(s, func):
     return get_current_fig_manager().canvas.mpl_connect(s, func)
 if FigureCanvasBase.mpl_connect.__doc__ is not None:
-    connect.__doc__ = FigureCanvasBase.mpl_connect.__doc__
+    connect.__doc__ = _shift_string(FigureCanvasBase.mpl_connect.__doc__)
 
 def disconnect(cid):
     return get_current_fig_manager().canvas.mpl_disconnect(cid)
 if FigureCanvasBase.mpl_disconnect.__doc__ is not None:
-    disconnect.__doc__ = FigureCanvasBase.mpl_disconnect.__doc__
+    disconnect.__doc__ = _shift_string(FigureCanvasBase.mpl_disconnect.__doc__)
 
 def get_plot_commands(): return ( 'axes', 'axis', 'bar', 'boxplot', 'cla', 'clf',
     'close', 'colorbar', 'cohere', 'csd', 'draw', 'errorbar',
@@ -492,7 +546,6 @@ def raise_msg_to_str(msg):
 
 
 
-
 def axis(*v, **kwargs):
     """
     Set/Get the axis properties::
@@ -504,9 +557,22 @@ def axis(*v, **kwargs):
 
         axis('off') turns off the axis lines and labels
 
-        axis('equal') sets the xlim width and ylim height to be to be
-            identical.  The longer of the two intervals is chosen
+        axis('equal') changes limits of x or y axis such that equal
+          tick mark increments are equal in size. This makes a
+          circle look like a circle, for example. This is persistent.
+          For example, when axis limits are changed after this command,
+          the scale remains equal
 
+        axis('scaled') makes scale equal, changes lengths of axes while
+          keeping limits of x and y axes fixed. Keeps lower left hand corner
+          in original position. Fixes axis limits.
+
+        axis('tight') changes limits x and y axis such that all data is
+          shown. If all data is already shown, it will move it to the center
+          of the figure without modifying (xmax-xmin) or (ymax-ymin). Note
+          this is slightly different than in matlab. Fixes axis limits.
+
+        axis('normal') sets the axis to normal, i.e. turns equal scale off
 
        if len(*v)==0, you can pass in xmin, xmax, ymin, ymax as kwargs
        selectively to alter just those limits w/o changing the others.
@@ -515,47 +581,49 @@ def axis(*v, **kwargs):
        The xmin, xmax, ymin, ymax tuple is returned
 
     """
-
+    ax = gca()
     if len(v)==1 and is_string_like(v[0]):
         s = v[0]
-        if s.lower()=='on': gca().set_axis_on()
-        elif s.lower()=='off': gca().set_axis_off()
+        if s.lower()=='on': ax.set_axis_on()
+        elif s.lower()=='off': ax.set_axis_off()
         elif s.lower()=='equal':
-            ax = gca()
-            xmin, xmax = ax.get_xlim()
-            ymin, ymax = ax.get_ylim()
-
-            width = xmax-xmin
-            height = ymax-ymin
-            # TODO: handle decreasing lim
-
-            interval = max([width, height])
-            ax.set_xlim((xmin, xmin+interval))
-            ax.set_ylim((ymin, ymin+interval))
+            ax.set_aspect('equal')
             draw_if_interactive()
-
+        elif s.lower()=='tight':
+            ax.autoscale_view()
+            ax.set_autoscale_on(False)
+            draw_if_interactive()
+        elif s.lower()=='scaled':
+            ax.set_autoscale_on(False)
+            ax.set_aspect('scaled',True)
+            draw_if_interactive()
+        elif s.lower()=='normal':
+            ax.set_autoscale_on(True)
+            ax.set_aspect('normal')
         else:
             raise ValueError('Unrecognized string %s to axis; try on or off' % s)
-        ax  = gca()
-        xmin, xmax = ax.get_xlim() 
+        xmin, xmax = ax.get_xlim()
         ymin, ymax = ax.get_ylim()
         draw_if_interactive()
         return xmin, xmax, ymin, ymax
 
     try: v[0]
     except IndexError:
-        xmin, xmax = gca().set_xlim(**kwargs)
-        ymin, ymax = gca().set_ylim(**kwargs)
+        xmin, xmax = ax.set_xlim(**kwargs)
+        ymin, ymax = ax.set_ylim(**kwargs)
         draw_if_interactive()
         return [xmin, xmax, ymin, ymax]
 
     v = v[0]
     if len(v) != 4:
         raise ValueError('v must contain [xmin xmax ymin ymax]')
-    
-    
-    gca().set_xlim([v[0], v[1]])
-    gca().set_ylim([v[2], v[3]])
+
+
+    ax.set_xlim([v[0], v[1]])
+    ax.set_ylim([v[2], v[3]])
+    if ax.get_aspect() == 'equal':
+        ax.set_aspect( 'equal', True )
+
     draw_if_interactive()
     return v
 
@@ -578,6 +646,7 @@ def axes(*args, **kwargs):
       frameon=False  : don't display the frame
       sharex=otherax : the current axes shares xaxis attribute with otherax
       sharey=otherax : the current axes shares yaxis attribute with otherax
+      polar=True|False : use a polar axes or not
 
     Examples
 
@@ -708,7 +777,7 @@ def figtext(*args, **kwargs):
     draw_if_interactive()
     return ret
 if Figure.text.__doc__ is not None:
-    figtext.__doc__ = Figure.text.__doc__
+    figtext.__doc__ = _shift_string(Figure.text.__doc__)
 
 def figimage(*args, **kwargs):
     # allow callers to override the hold state by passing hold=True|False
@@ -717,7 +786,7 @@ def figimage(*args, **kwargs):
     gci._current = ret
     return ret
 if Figure.figimage.__doc__ is not None:
-    figimage.__doc__ = Figure.figimage.__doc__ + """
+    figimage.__doc__ = _shift_string(Figure.figimage.__doc__) + """
 Addition kwargs: hold = [True|False] overrides default hold state"""
 
 def figlegend(handles, labels, loc, **kwargs):
@@ -744,7 +813,7 @@ def savefig(*args, **kwargs):
     fig = gcf()
     return fig.savefig(*args, **kwargs)
 if Figure.savefig.__doc__ is not None:
-    savefig.__doc__ = Figure.savefig.__doc__
+    savefig.__doc__ = _shift_string(Figure.savefig.__doc__)
 
 
 def figure(num=None, # autoincrement if None, else integer from 1-N
@@ -797,7 +866,8 @@ def figure(num=None, # autoincrement if None, else integer from 1-N
         figManager = new_figure_manager(num, figsize, dpi, facecolor, edgecolor, frameon)
         _pylab_helpers.Gcf.set_active(figManager)
         figManager.canvas.figure.number = num
-        
+
+    draw_if_interactive()
     return figManager.canvas.figure
 
 def gca(**kwargs):
@@ -877,7 +947,7 @@ def isinteractive():
 def imread(*args, **kwargs):
     return _imread(*args, **kwargs)
 if _imread.__doc__ is not None:
-    imread.__doc__ = _imread.__doc__
+    imread.__doc__ = _shift_string(_imread.__doc__)
 
 
 def load(fname,comments='%',delimiter=None):
@@ -909,9 +979,9 @@ def load(fname,comments='%',delimiter=None):
 
     comments is the character used to indicate the start of a comment
     in the file
-    
+
     delimiter is a string-like character used to seperate values in the
-    file. If delimiter is unspecified or none, any whitespace string is 
+    file. If delimiter is unspecified or none, any whitespace string is
     a separator.
 
     """
@@ -947,13 +1017,13 @@ def load(fname,comments='%',delimiter=None):
 def rc(*args, **kwargs):
     matplotlib.rc(*args, **kwargs)
 if matplotlib.rc.__doc__ is not None:
-    rc.__doc__ =   matplotlib.rc.__doc__
+    rc.__doc__ =  _shift_string(matplotlib.rc.__doc__)
 
 def rcdefaults():
     matplotlib.rcdefaults()
     draw_if_interactive()
 if matplotlib.rcdefaults.__doc__ is not None:
-    rcdefaults.__doc__ =   matplotlib.rcdefaults.__doc__
+    rcdefaults.__doc__ =   _shift_string(matplotlib.rcdefaults.__doc__)
 
 def save(fname, X, fmt='%.18e'):
     """
@@ -1061,7 +1131,7 @@ def twinx(ax=None):
     ax2 = gcf().add_axes(ax.get_position(), sharex=ax, frameon=False)
     ax2.yaxis.tick_right()
     ax2.yaxis.set_label_position('right')
-
+    ax.yaxis.tick_left()
     draw_if_interactive()
     return ax2
 
@@ -1481,13 +1551,6 @@ def matshow(*args,**kw):
     else:
         return fig
 
-def set(*args, **kwargs):
-    message = 'set deprecated because it overrides python2.4 builtin set.  Use setp'
-    warnings.warn(message, DeprecationWarning, stacklevel=2)
-
-    return setp(*args, **kwargs)
-
-
 
 def setp(*args, **kwargs):
     ret = _setp(*args, **kwargs)
@@ -1496,6 +1559,52 @@ def setp(*args, **kwargs):
 if _setp.__doc__ is not None:
     setp.__doc__ = _setp.__doc__
 
+
+def subplots_adjust(*args, **kwargs):
+    """
+    subplots_adjust(left=None, right=None, bottom=None, top=None, wspace=0.2, hspace=0.2)
+
+    Tune the subplot layout via the figure.SubplotParams mechanism.
+    The parameter meanings (and suggested defaults) are
+
+      left  = 0.125  # the left side of the subplots of the figure
+      right = 0.9    # the right side of the subplots of the figure
+      bottom = 0.1   # the bottom of the subplots of the figure
+      top = 0.9      # the top of the subplots of the figure
+      wspace = 0.2   # the amount of width reserved for blank space between subplots
+      hspace = 0.2   # the amount of height reserved for white space between subplots
+
+    The actual defaults are controlled by the rc file
+    """
+    fig = gcf()
+    fig.subplots_adjust(*args, **kwargs)
+    draw_if_interactive()
+
+
+def subplot_tool(targetfig=None):
+    """
+    Launch a subplot tool window for targetfig (default gcf)
+
+    A matplotlib.widgets.SubplotTool instance is returned
+    """
+    tbar = rcParams['toolbar'] # turn off the navigation toolbar for the toolfig
+    rcParams['toolbar'] = 'None'
+    if targetfig is None:
+        manager = get_current_fig_manager()
+        targetfig = manager.canvas.figure
+    else:
+        # find the manager for this figure
+        for manager in _pylab_helpers.Gcf._activeQue:
+            if manager.canvas.figure==targetfig: break
+        else: raise RuntimeError('Could not find manager for targetfig')
+
+    toolfig = figure(figsize=(6,3))
+    toolfig.subplots_adjust(top=0.9)
+    ret =  SubplotTool(targetfig, toolfig)
+    rcParams['toolbar'] = tbar
+    _pylab_helpers.Gcf.set_active(manager)  # restore the current figure
+    return ret
+
 ### The following functions were autogenerated by the boilerplate.py
 ### script.  They are simple wrappers around the Axes methods of the
 
@@ -1513,7 +1622,7 @@ def axhline(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.axhline.__doc__ is not None:
@@ -1534,7 +1643,7 @@ def axhspan(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.axhspan.__doc__ is not None:
@@ -1555,7 +1664,7 @@ def axvline(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.axvline.__doc__ is not None:
@@ -1576,7 +1685,7 @@ def axvspan(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.axvspan.__doc__ is not None:
@@ -1597,7 +1706,7 @@ def bar(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.bar.__doc__ is not None:
@@ -1618,7 +1727,7 @@ def barh(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.barh.__doc__ is not None:
@@ -1639,7 +1748,7 @@ def boxplot(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.boxplot.__doc__ is not None:
@@ -1660,7 +1769,7 @@ def cohere(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.cohere.__doc__ is not None:
@@ -1681,7 +1790,6 @@ def clabel(*args, **kwargs):
     except:
         hold(b)
         raise
-    if ret.mappable is not None: gci._current = ret.mappable
     hold(b)
     return ret
 if Axes.clabel.__doc__ is not None:
@@ -1702,7 +1810,7 @@ def contour(*args, **kwargs):
     except:
         hold(b)
         raise
-    if ret[1].mappable is not None: gci._current = ret[1].mappable
+    if ret._A is not None: gci._current = ret
     hold(b)
     return ret
 if Axes.contour.__doc__ is not None:
@@ -1723,7 +1831,7 @@ def contourf(*args, **kwargs):
     except:
         hold(b)
         raise
-    if ret[1].mappable is not None: gci._current = ret[1].mappable
+    if ret._A is not None: gci._current = ret
     hold(b)
     return ret
 if Axes.contourf.__doc__ is not None:
@@ -1744,7 +1852,7 @@ def csd(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.csd.__doc__ is not None:
@@ -1765,7 +1873,7 @@ def errorbar(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.errorbar.__doc__ is not None:
@@ -1786,7 +1894,7 @@ def fill(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.fill.__doc__ is not None:
@@ -1807,7 +1915,7 @@ def hist(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.hist.__doc__ is not None:
@@ -1828,7 +1936,7 @@ def hlines(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.hlines.__doc__ is not None:
@@ -1870,7 +1978,7 @@ def loglog(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.loglog.__doc__ is not None:
@@ -1912,7 +2020,7 @@ def pcolor_classic(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.pcolor_classic.__doc__ is not None:
@@ -1933,7 +2041,7 @@ def pie(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.pie.__doc__ is not None:
@@ -1954,7 +2062,7 @@ def plot(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.plot.__doc__ is not None:
@@ -1975,7 +2083,7 @@ def plot_date(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.plot_date.__doc__ is not None:
@@ -1996,7 +2104,7 @@ def psd(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.psd.__doc__ is not None:
@@ -2038,7 +2146,7 @@ def scatter_classic(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.scatter_classic.__doc__ is not None:
@@ -2059,7 +2167,7 @@ def semilogx(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.semilogx.__doc__ is not None:
@@ -2080,7 +2188,7 @@ def semilogy(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.semilogy.__doc__ is not None:
@@ -2122,7 +2230,7 @@ def spy(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.spy.__doc__ is not None:
@@ -2164,7 +2272,7 @@ def stem(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.stem.__doc__ is not None:
@@ -2185,7 +2293,7 @@ def vlines(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.vlines.__doc__ is not None:
@@ -2206,7 +2314,7 @@ def quiver(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.quiver.__doc__ is not None:
